@@ -343,10 +343,7 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
     QString fileNameIn2;
     QString fileNameOut2;
 
-    if(!m_fileAccess.isValid())
-        return mErrors;
-
-    if(!m_fileAccess.isNormal())
+    if(m_fileAccess.isValid() && !m_fileAccess.isNormal())
     {
         mErrors.append(i18n("%1 is not a normal file.", m_fileAccess.prettyAbsPath()));
         return mErrors;
@@ -912,7 +909,7 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, qint64& ski
         if(buf[0] == '\xEF' && buf[1] == '\xBB' && buf[2] == '\xBF')
         {
             skipBytes = 3;
-            return QTextCodec::codecForName("UTF-8-BOM");
+            return QTextCodec::codecForName("UTF-8");
         }
     }
     skipBytes = 0;
