@@ -20,6 +20,8 @@
  */
 #include "Utils.h"
 
+#include "fileaccess.h"
+
 #include <QString>
 #include <QStringList>
 #include <QHash>
@@ -123,3 +125,19 @@ bool Utils::wildcardMultiMatch(const QString& wildcard, const QString& testStrin
 
     return false;
 }
+
+/*
+    QUrl::toLocalFile does some special handling for locally visable windows network drives.
+    If QUrl::isLocal however it returns false we get an empty string back.
+*/
+QString Utils::urlToString(const QUrl &url)
+{
+    if(!FileAccess::isLocal(url))
+        return url.toString();
+
+    QString result = url.toLocalFile();
+    if(result.isEmpty())
+        return url.path();
+
+    return result;
+}
