@@ -18,6 +18,7 @@
 #include "options.h"
 #include "FileNameLineEdit.h"
 #include "RLPainter.h"
+#include "Utils.h"
 
 #include <algorithm>
 #include <cmath>
@@ -136,9 +137,9 @@ class DiffTextWindowData
         RLPainter& p, const LineData* pld,
         const DiffList* pLineDiff1, const DiffList* pLineDiff2, int line,
         int whatChanged, int whatChanged2, int srcLineIdx,
-        int wrapLineOffset, int wrapLineLength, bool bWrapLine, const QRect& invalidRect, int deviceWidth);
+        int wrapLineOffset, int wrapLineLength, bool bWrapLine, const QRect& invalidRect);
 
-    void draw(RLPainter& p, const QRect& invalidRect, int deviceWidth, int beginLine, int endLine);
+    void draw(RLPainter& p, const QRect& invalidRect, int beginLine, int endLine);
 
     QStatusBar* m_pStatusBar;
 
@@ -288,19 +289,19 @@ int DiffTextWindow::getFirstLine()
 
 void DiffTextWindow::setHorizScrollOffset(int horizScrollOffset)
 {
-    int fontWidth = fontMetrics().width('0');
+    int fontWidth = Utils::getHorizontalAdvance(fontMetrics(), '0');
     int xOffset = d->leftInfoWidth() * fontWidth;
 
     int deltaX = d->m_horizScrollOffset - std::max(0, horizScrollOffset);
 
     d->m_horizScrollOffset = std::max(0, horizScrollOffset);
 
-    QRect r(xOffset, 0, width() - xOffset, height());
+    QRect r(xOffset, 0, width(), height());
 
     if(d->m_pOptions->m_bRightToLeftLanguage)
     {
         deltaX = -deltaX;
-        r = QRect(width() - xOffset - 2, 0, -(width() - xOffset), height()).normalized();
+        r = QRect(width() - xOffset - 2, 0, -(width()), height()).normalized();
     }
 
     if(d->m_bSelectionInProgress && d->m_selection.isValidFirstLine())
@@ -313,6 +314,7 @@ void DiffTextWindow::setHorizScrollOffset(int horizScrollOffset)
     else
     {
         scroll(deltaX, 0, r);
+        update();
     }
 }
 
@@ -433,7 +435,7 @@ void DiffTextWindow::mousePressEvent(QMouseEvent* e)
         int pos;
         convertToLinePos(e->x(), e->y(), line, pos);
 
-        int fontWidth = fontMetrics().width('0');
+        int fontWidth = Utils::getHorizontalAdvance(fontMetrics(), '0');
         int xOffset = d->leftInfoWidth() * fontWidth;
 
         if((!d->m_pOptions->m_bRightToLeftLanguage && e->x() < xOffset) || (d->m_pOptions->m_bRightToLeftLanguage && e->x() > width() - xOffset))
@@ -563,7 +565,7 @@ void DiffTextWindow::mouseMoveEvent(QMouseEvent* e)
 
         // Scroll because mouse moved out of the window
         const QFontMetrics& fm = fontMetrics();
-        int fontWidth = fm.width('0');
+        int fontWidth = Utils::getHorizontalAdvance(fm, '0');
         int deltaX = 0;
         int deltaY = 0;
         if(!d->m_pOptions->m_bRightToLeftLanguage)
@@ -756,7 +758,7 @@ void DiffTextWindowData::prepareTextLayout(QTextLayout& textLayout, bool /*bFirs
     int leading = m_pDiffTextWindow->fontMetrics().leading();
     int height = 0;
 
-    int fontWidth = m_pDiffTextWindow->fontMetrics().width('0');
+    int fontWidth = Utils::getHorizontalAdvance(m_pDiffTextWindow->fontMetrics(), '0');
     int xOffset = leftInfoWidth() * fontWidth - m_horizScrollOffset;
     int textWidth = visibleTextWidth;
     if(textWidth < 0)
@@ -793,6 +795,10 @@ void DiffTextWindowData::prepareTextLayout(QTextLayout& textLayout, bool /*bFirs
         textLayout.setPosition(QPointF(xOffset, 0));
 }
 
+/*
+    Don't try to use invalid rect to block drawing of lines based on there apparent dementions.
+    This does not always work for very long lines being scrolled horizontally. (Causes blanking of diff text area)
+*/
 void DiffTextWindowData::writeLine(
     RLPainter& p,
     const LineData* pld,
@@ -805,25 +811,18 @@ void DiffTextWindowData::writeLine(
     int wrapLineOffset,
     int wrapLineLength,
     bool bWrapLine,
-    const QRect& invalidRect,
-    int deviceWidth)
+    const QRect& invalidRect)
 {
     QFont normalFont = p.font();
 
     const QFontMetrics& fm = p.fontMetrics();
     int fontHeight = fm.lineSpacing();
     int fontAscent = fm.ascent();
-    int fontWidth = fm.width('0');
+    int fontWidth = Utils::getHorizontalAdvance(fm, '0');
 
     int xOffset = leftInfoWidth() * fontWidth - m_horizScrollOffset;
     int yOffset = (line - m_firstLine) * fontHeight;
 
-    QRect lineRect(xOffset, yOffset, deviceWidth, fontHeight);
-    if(!invalidRect.intersects(lineRect))
-    {
-        return;
-    }
-
     int fastSelectorLine1 = m_pDiffTextWindow->convertDiff3LineIdxToLine(m_fastSelectorLine1);
     int fastSelectorLine2 = m_pDiffTextWindow->convertDiff3LineIdxToLine(m_fastSelectorLine1 + m_fastSelectorNofLines) - 1;
 
@@ -960,8 +959,8 @@ void DiffTextWindowData::writeLine(
     {
         if(m_pOptions->m_bShowLineNumbers && !bWrapLine)
         {
-            QString num;
-            num.sprintf("%0*d", m_lineNumberWidth, srcLineIdx + 1);
+            QString num = QString::number(srcLineIdx + 1);
+            Q_ASSERT(!num.isEmpty());
             p.drawText(0, yOffset + fontAscent, num);
             //p.drawLine( xLeft -1, yOffset, xLeft -1, yOffset+fontHeight-1 );
         }
@@ -1027,12 +1026,12 @@ void DiffTextWindow::paintEvent(QPaintEvent* e)
 
     int endLine = std::min(d->m_firstLine + getNofVisibleLines() + 2, getNofLines());
 
-    RLPainter p(this, d->m_pOptions->m_bRightToLeftLanguage, width(), fontMetrics().width('0'));
+    RLPainter p(this, d->m_pOptions->m_bRightToLeftLanguage, width(), Utils::getHorizontalAdvance(fontMetrics(), '0'));
 
     p.setFont(font());
     p.QPainter::fillRect(invalidRect, d->m_pOptions->m_bgColor);
 
-    d->draw(p, invalidRect, width(), d->m_firstLine, endLine);
+    d->draw(p, invalidRect, d->m_firstLine, endLine);
     p.end();
 
     d->m_oldFirstLine = d->m_firstLine;
@@ -1053,12 +1052,12 @@ void DiffTextWindow::print(RLPainter& p, const QRect&, int firstLine, int nofLin
     QRect invalidRect = QRect(0, 0, 1000000000, 1000000000);
     QColor bgColor = d->m_pOptions->m_bgColor;
     d->m_pOptions->m_bgColor = Qt::white;
-    d->draw(p, invalidRect, p.window().width(), firstLine, std::min(firstLine + nofLinesPerPage, getNofLines()));
+    d->draw(p, invalidRect, firstLine, std::min(firstLine + nofLinesPerPage, getNofLines()));
     d->m_pOptions->m_bgColor = bgColor;
     d->m_firstLine = oldFirstLine;
 }
 
-void DiffTextWindowData::draw(RLPainter& p, const QRect& invalidRect, int deviceWidth, int beginLine, int endLine)
+void DiffTextWindowData::draw(RLPainter& p, const QRect& invalidRect, int beginLine, int endLine)
 {
     m_lineNumberWidth = m_pOptions->m_bShowLineNumbers ? (int)log10((double)std::max(m_size, 1)) + 1 : 0;
 
@@ -1122,8 +1121,7 @@ void DiffTextWindowData::draw(RLPainter& p, const QRect& invalidRect, int device
             wrapLineOffset,
             wrapLineLength,
             bWrapLine,
-            invalidRect,
-            deviceWidth);
+            invalidRect);
     }
 }
 
@@ -1182,7 +1180,7 @@ void DiffTextWindow::resizeEvent(QResizeEvent* e)
     QSize s = e->size();
     QFontMetrics fm = fontMetrics();
     int visibleLines = s.height() / fm.lineSpacing() - 2;
-    int visibleColumns = s.width() / fm.width('0') - d->leftInfoWidth();
+    int visibleColumns = s.width() / Utils::getHorizontalAdvance(fm, '0') - d->leftInfoWidth();
     if(e->size().height() != e->oldSize().height())
         emit resizeHeightChangedSignal(visibleLines);
     if(e->size().width() != e->oldSize().width())
@@ -1201,7 +1199,7 @@ int DiffTextWindow::getNofVisibleLines()
 int DiffTextWindow::getVisibleTextAreaWidth()
 {
     QFontMetrics fm = fontMetrics();
-    return width() - d->leftInfoWidth() * fm.width('0');
+    return width() - d->leftInfoWidth() * Utils::getHorizontalAdvance(fm, '0');
 }
 
 QString DiffTextWindow::getSelection()
@@ -1448,7 +1446,7 @@ void DiffTextWindow::convertSelectionToD3LCoords()
     d->m_selection.end(lastD3LIdx, lastD3LPos);
 }
 
-int s_maxNofRunnables = 0;
+QAtomicInt s_maxNofRunnables = 0;
 
 class RecalcWordWrapRunnable : public QRunnable
 {
@@ -1576,7 +1574,7 @@ void DiffTextWindow::recalcWordWrapHelper(int wrapLineVectorSize, int visibleTex
         if(visibleTextWidth < 0)
             visibleTextWidth = getVisibleTextAreaWidth();
         else
-            visibleTextWidth -= d->leftInfoWidth() * fontMetrics().width('0');
+            visibleTextWidth -= d->leftInfoWidth() * Utils::getHorizontalAdvance(fontMetrics(), '0');
         int i;
         int wrapLineIdx = 0;
         int size = d->m_pDiff3LineVector->size();
@@ -1777,7 +1775,8 @@ DiffTextWindowFrame::DiffTextWindowFrame(QWidget* pParent, QStatusBar* pStatusBa
     pHL2->setMargin(0);
     pHL2->setSpacing(2);
     pHL2->addWidget(d->m_pTopLine, 0);
-    d->m_pEncoding = new EncodingLabel(i18n("Encoding:"), this, psd, pOptions);
+    d->m_pEncoding = new EncodingLabel(i18n("Encoding:"), psd, pOptions);
+    connect((EncodingLabel*) d->m_pEncoding, &EncodingLabel::encodingChanged, this, &DiffTextWindowFrame::slotEncodingChanged);
 
     d->m_pLineEndStyle = new QLabel(i18n("Line end style:"));
     pHL2->addWidget(d->m_pEncoding);
@@ -1837,8 +1836,7 @@ void DiffTextWindowFrame::setFirstLine(int firstLine)
 
         int l = pDTW->calcTopLineInFile(firstLine);
 
-        int w = d->m_pTopLine->fontMetrics().width(
-            s + ' ' + QString().fill('0', lineNumberWidth));
+        int w = Utils::getHorizontalAdvance(d->m_pTopLine->fontMetrics(), s + ' ' + QString().fill('0', lineNumberWidth));
         d->m_pTopLine->setMinimumWidth(w);
 
         if(l == -1)
@@ -1908,15 +1906,9 @@ void DiffTextWindowFrame::slotBrowseButtonClicked()
     }
 }
 
-void DiffTextWindowFrame::sendEncodingChangedSignal(QTextCodec* c)
-{
-    emit encodingChanged(c);
-}
-
-EncodingLabel::EncodingLabel(const QString& text, DiffTextWindowFrame* pDiffTextWindowFrame, SourceData* pSD, Options* pOptions)
+EncodingLabel::EncodingLabel(const QString& text, SourceData* pSD, Options* pOptions)
     : QLabel(text)
 {
-    m_pDiffTextWindowFrame = pDiffTextWindowFrame;
     m_pOptions = pOptions;
     m_pSourceData = pSD;
     m_pContextEncodingMenu = nullptr;
@@ -1983,19 +1975,21 @@ void EncodingLabel::insertCodec(const QString& visibleCodecName, QTextCodec* pCo
     if(pCodec != nullptr && !codecEnumList.contains(CodecMIBEnum))
     {
         QAction* pAction = new QAction(pMenu); // menu takes ownership, so deleting the menu deletes the action too.
-        QLatin1String codecName(pCodec->name());
+        QByteArray nameArray = pCodec->name();
+        QLatin1String codecName = QLatin1String(nameArray);
+
         pAction->setText(visibleCodecName.isEmpty() ? codecName : visibleCodecName + QLatin1String(" (") + codecName + QLatin1String(")"));
         pAction->setData(CodecMIBEnum);
         pAction->setCheckable(true);
         if(currentTextCodecEnum == CodecMIBEnum)
             pAction->setChecked(true);
         pMenu->addAction(pAction);
-        connect(pAction, &QAction::triggered, this, &EncodingLabel::slotEncodingChanged);
+        connect(pAction, &QAction::triggered, this, &EncodingLabel::slotSelectEncoding);
         codecEnumList.append(CodecMIBEnum);
     }
 }
 
-void EncodingLabel::slotEncodingChanged()
+void EncodingLabel::slotSelectEncoding()
 {
     QAction* pAction = qobject_cast<QAction*>(sender());
     if(pAction)
@@ -2015,6 +2009,7 @@ void EncodingLabel::slotEncodingChanged()
                 recentEncodings.append(s);
             }
         }
-        m_pDiffTextWindowFrame->sendEncodingChangedSignal(pCodec);
+
+        emit encodingChanged(pCodec);
     }
 }
