@@ -116,7 +116,6 @@ public:
    DiffTextWindow* getDiffTextWindow();
    void init();
    void setFirstLine(int firstLine);
-   void sendEncodingChangedSignal(QTextCodec* c);
 Q_SIGNALS:
    void fileNameChanged(const QString&, e_SrcSelector);
    void encodingChanged(QTextCodec*);
@@ -126,7 +125,7 @@ protected:
 private Q_SLOTS:
    void slotReturnPressed();
    void slotBrowseButtonClicked();
-
+   void slotEncodingChanged(QTextCodec* c) { emit encodingChanged(c); };//relay signal from encoding label
  private:
    DiffTextWindowFrameData* d;
 };
@@ -135,14 +134,15 @@ class EncodingLabel : public QLabel
 {
    Q_OBJECT
 public:
-   EncodingLabel( const QString & text, DiffTextWindowFrame* pDiffTextWindowFrame, SourceData* psd, Options* pOptions);
+   EncodingLabel( const QString & text, SourceData* psd, Options* pOptions);
 protected:
    void mouseMoveEvent(QMouseEvent *ev) override;
    void mousePressEvent(QMouseEvent *ev) override;
+Q_SIGNALS:
+    void encodingChanged(QTextCodec*);
 private Q_SLOTS:
-   void slotEncodingChanged();
+   void slotSelectEncoding();
 private:
-   DiffTextWindowFrame* m_pDiffTextWindowFrame; //To send "EncodingChanged" signal
    QMenu* m_pContextEncodingMenu;
    SourceData* m_pSourceData; //SourceData to get access to "isEmpty()" and "isFromBuffer()" functions
    static const int m_maxRecentEncodings  = 5;
