@@ -84,8 +84,6 @@ void FileAccess::setFile(FileAccess* pParent, const QFileInfo& fi)
 
     m_fileInfo = fi;
     m_url = QUrl::fromLocalFile(m_fileInfo.filePath());
-    if(!m_url.scheme().isEmpty())
-        m_url.setScheme(QLatin1String("file"));
 
     m_pParent = pParent;
     loadData();
@@ -106,13 +104,14 @@ void FileAccess::setFile(const QUrl& url, bool bWantToWrite)
     Q_ASSERT(parent() == nullptr || url != parent()->url());
 
     m_url = url;
-    //QUrl::isLocalFile assumes the scheme is set.
-    if(m_url.scheme().isEmpty())
-        m_url.setScheme(QLatin1String("file"));
 
-    if(m_url.isLocalFile() || !m_url.isValid()) // Treat invalid urls as local files.
+    if(isLocal()) // Invalid urls are treated as local files.
     {
-        m_fileInfo.setFile(m_url.toLocalFile());
+        /*
+            Utils::urlToString handles choosing the right API from QUrl.
+        */
+        m_fileInfo.setFile(Utils::urlToString(url));
+        
         m_pParent = nullptr;
 
         loadData();
@@ -121,11 +120,10 @@ void FileAccess::setFile(const QUrl& url, bool bWantToWrite)
     {
         m_name = m_url.fileName();
         FileAccessJobHandler jh(this);            // A friend, which writes to the parameters of this class!
-        jh.stat(2 /*all details*/, bWantToWrite); // returns bSuccess, ignored
-
-        m_bValidData = true; // After running stat() the variables are initialised
-                             // and valid even if the file doesn't exist and the stat
-                             // query failed.
+        if(jh.stat(2 /*all details*/, bWantToWrite))
+            m_bValidData = true; // After running stat() the variables are initialised
+                                // and valid even if the file doesn't exist and the stat
+                                // query failed.
     }
 }
 
@@ -179,13 +177,16 @@ void FileAccess::loadData()
     m_bValidData = true;
 }
 
-void FileAccess::addPath(const QString& txt)
+void FileAccess::addPath(const QString& txt, bool reinit)
 {
     if(!isLocal() && m_url.isValid())
     {
         QUrl url = m_url.adjusted(QUrl::StripTrailingSlash);
         url.setPath(url.path() + '/' + txt);
-        setFile(url); // reinitialise
+        m_url = url;
+
+        if(reinit)
+            setFile(url); // reinitialize
     }
     else
     {
@@ -258,11 +259,25 @@ void FileAccess::setFromUdsEntry(const KIO::UDSEntry& e, FileAccess *parent)
             }
             case KIO::UDSEntry::UDS_FILE_TYPE:
             {
-                fileType = e.numberValue(f);
-                m_bDir = (fileType & QT_STAT_MASK) == QT_STAT_DIR;
-                m_bFile = (fileType & QT_STAT_MASK) == QT_STAT_REG;
-                m_bSymLink = (fileType & QT_STAT_MASK) == QT_STAT_LNK;
-                m_bExists = fileType != 0;
+                /*
+                    According to KIO docs UDS_LINK_DEST not S_ISLNK should be used to determine if the url is a symlink.
+                    UDS_FILE_TYPE is explicitly stated to be the type of the linked file not the link itself.
+                */
+                if(!e.isLink())
+                {
+                    fileType = e.numberValue(f);
+                    m_bDir = (fileType & QT_STAT_MASK) == QT_STAT_DIR;
+                    m_bFile = (fileType & QT_STAT_MASK) == QT_STAT_REG;
+                    m_bSymLink = (fileType & QT_STAT_MASK) == QT_STAT_LNK;
+                }
+                else
+                {
+                    m_bDir = false;
+                    m_bFile = false;
+                    m_bSymLink = true;
+                }
+                    
+                m_bExists = m_bSymLink || fileType != 0;
                 //m_fileType = fileType;
                 break;
             }
@@ -281,17 +296,9 @@ void FileAccess::setFromUdsEntry(const KIO::UDSEntry& e, FileAccess *parent)
         }
     }
 
-    m_fileInfo = QFileInfo(filePath);
-    m_fileInfo.setCaching(true);
     if(m_url.isEmpty())
     {
-        if(parent != nullptr)
-        {
-            m_url = parent->url().resolved(QUrl(filePath));
-            //Verify that the scheme doesn't change.
-            Q_ASSERT(m_url.scheme() == parent->url().scheme());
-        }
-        else
+        if(parent == nullptr)
         {
             /*
              Invalid entry we don't know the full url because KIO didn't tell us and there is no parent
@@ -301,13 +308,36 @@ void FileAccess::setFromUdsEntry(const KIO::UDSEntry& e, FileAccess *parent)
             qCWarning(kdiffFileAccess) << i18n("Unable to determine full url. No parent specified.");
             return;
         }
+        /*
+            Don't trust QUrl::resolved it doesn't always do what kdiff3 wants.
+        */
+        m_url = parent->url();
+        addPath(filePath, false);
+        //Not something I expect to happen but can't rule it out either
+        if(Q_UNLIKELY(m_url == parent->url()))
+        {
+            m_url.clear();
+            qCritical() << "Parent and child could not be distingished.";
+            return;
+        }
+
+        //Verify that the scheme doesn't change.
+        Q_ASSERT(m_url.scheme() == parent->url().scheme());
     }
 
-    m_name = m_fileInfo.fileName();
-    if(isLocal() && m_name.isEmpty())
+    //KIO does this when stating a remote folder.
+    if(filePath.isEmpty())
     {
-        m_name = m_fileInfo.absoluteDir().dirName();
+        filePath = m_url.path();
     }
+
+    m_fileInfo = QFileInfo(filePath);
+    m_fileInfo.setCaching(true);
+
+    m_name = m_fileInfo.fileName();
+    if(m_name.isEmpty())
+        m_name = m_fileInfo.absoluteDir().dirName();
+    
     if(isLocal())
         m_bExists = m_fileInfo.exists();
 
@@ -377,16 +407,17 @@ QUrl FileAccess::url() const
 {
     QUrl url = m_url;
 
-    if(url.isLocalFile())
+    if(isLocal())
     {
         url = QUrl::fromLocalFile(absoluteFilePath());
     }
     return url;
 }
 
+//Workaround for QUrl::isLocalFile behavior that does not fit KDiff3's expectations.
 bool FileAccess::isLocal() const
 {
-    return m_url.isLocalFile() || !m_url.isValid();
+    return m_url.isLocalFile() || !m_url.isValid() || m_url.scheme().isEmpty();
 }
 
 bool FileAccess::isReadable() const
@@ -448,9 +479,31 @@ QString FileAccess::fileName(bool needTmp) const
 
 QString FileAccess::fileRelPath() const
 {
-    QString path = m_baseDir.relativeFilePath(m_fileInfo.absoluteFilePath());
+    QString path;
 
-    return path;
+    if(isLocal())
+    {
+        path = m_baseDir.relativeFilePath(m_fileInfo.absoluteFilePath());
+
+        return path;
+    }
+    else
+    {
+        //Stop right before the root directory
+        if(parent() == nullptr) return QString();
+
+        const FileAccess *curEntry = this;
+        path = fileName();
+        //Avoid recursing to FileAccess::fileRelPath or we can get very large stacks.
+        curEntry = curEntry->parent();
+        while(curEntry != nullptr)
+        {
+            if(curEntry->parent())
+                path.prepend(curEntry->fileName() + "/");
+            curEntry= curEntry->parent();
+        }
+        return path;
+    }    
 }
 
 FileAccess* FileAccess::parent() const
@@ -458,6 +511,7 @@ FileAccess* FileAccess::parent() const
     return m_pParent;
 }
 
+//Workaround for QUrl::toDisplayString/QUrl::toString behavior that does not fit KDiff3's expectations
 QString FileAccess::prettyAbsPath() const
 {
     return isLocal() ? absoluteFilePath() : m_url.toDisplayString();
@@ -767,7 +821,7 @@ QString FileAccess::cleanPath(const QString& path) // static
     FileAccess fa(path);
     if(fa.isLocal())
     {
-        return QDir().cleanPath(path);
+        return QDir::cleanPath(path);
     }
     else
     {
@@ -804,6 +858,7 @@ bool FileAccess::createBackup(const QString& bakExtension)
 
 void FileAccess::doError()
 {
+    m_bValidData = true;
     m_bExists = false;
 }
 
@@ -872,11 +927,20 @@ bool FileAccessJobHandler::stat(short detail, bool bWantToWrite)
 
 void FileAccessJobHandler::slotStatResult(KJob* pJob)
 {
-    if(pJob->error())
+    int err = pJob->error();
+    if(err != KJob::NoError)
     {
-        //pJob->uiDelegate()->showErrorMessage();
-        m_pFileAccess->doError();
-        m_bSuccess = true;
+        if(err != KIO::ERR_DOES_NOT_EXIST)
+        {
+            pJob->uiDelegate()->showErrorMessage();
+            m_bSuccess = false;
+            m_pFileAccess->reset();
+        }
+        else
+        {
+            m_pFileAccess->doError();
+            m_bSuccess = true;
+        }
     }
     else
     {
@@ -1052,7 +1116,7 @@ bool FileAccessJobHandler::removeFile(const QUrl& fileName)
         connect(pJob, &KIO::SimpleJob::result, this, &FileAccessJobHandler::slotSimpleJobResult);
         connect(pJob, &KIO::SimpleJob::finished, this, &FileAccessJobHandler::slotJobEnded);
 
-        ProgressProxy::enterEventLoop(pJob, i18n("Removing file: %1", fileName.toDisplayString()));
+        ProgressProxy::enterEventLoop(pJob, i18n("Removing file: %1", FileAccess::prettyAbsPath(fileName)));
         return m_bSuccess;
     }
 }
@@ -1069,7 +1133,7 @@ bool FileAccessJobHandler::symLink(const QUrl& linkTarget, const QUrl& linkLocat
         connect(pJob, &KIO::CopyJob::finished, this, &FileAccessJobHandler::slotJobEnded);
 
         ProgressProxy::enterEventLoop(pJob,
-                                      i18n("Creating symbolic link: %1 -> %2", linkLocation.toDisplayString(), linkTarget.toDisplayString()));
+                                      i18n("Creating symbolic link: %1 -> %2", FileAccess::prettyAbsPath(linkLocation), FileAccess::prettyAbsPath(linkTarget)));
         return m_bSuccess;
     }
 }
@@ -1175,8 +1239,8 @@ bool FileAccessJobHandler::listDir(t_DirectoryList* pDirList, bool bRecursive, b
         if(fiList.isEmpty())
         {
             /*
-                Sadly Qt provides no error information making this case ambigious.
-                A readablity check is the best we can do.
+                Sadly Qt provides no error information making this case ambiguous.
+                A readability check is the best we can do.
             */
             m_bSuccess = dir.isReadable();
         }
