@@ -142,7 +142,7 @@ void GnuDiff::find_and_hash_each_line(struct file_data *current)
     LineRef alloc_lines = current->alloc_lines;
     LineRef line = 0;
     LineRef linbuf_base = current->linbuf_base;
-    LineRef *cureqs = (LineRef *)xmalloc(alloc_lines * sizeof *cureqs);
+    LineRef *cureqs = (LineRef *)xmalloc(alloc_lines * sizeof(*cureqs));
     struct equivclass *eqs = equivs;
     LineRef eqs_index = equivs_index;
     LineRef eqs_alloc = equivs_alloc;
@@ -212,10 +212,10 @@ void GnuDiff::find_and_hash_each_line(struct file_data *current)
                 i = eqs_index++;
                 if(i == eqs_alloc)
                 {
-                    if((LineRef)(LINEREF_MAX / (2 * sizeof *eqs)) <= eqs_alloc)
+                    if((LineRef)(LINEREF_MAX / (2 * sizeof(*eqs))) <= eqs_alloc)
                         xalloc_die();
                     eqs_alloc *= 2;
-                    eqs = (equivclass *)xrealloc(eqs, eqs_alloc * sizeof *eqs);
+                    eqs = (equivclass *)xrealloc(eqs, eqs_alloc * sizeof(*eqs));
                 }
                 eqs[i].next = *bucket;
                 eqs[i].hash = h;
@@ -251,13 +251,13 @@ void GnuDiff::find_and_hash_each_line(struct file_data *current)
         if(line == alloc_lines)
         {
             /* Double (alloc_lines - linbuf_base) by adding to alloc_lines.  */
-            if((LineRef)(LINEREF_MAX / 3) <= alloc_lines || (LineRef)(LINEREF_MAX / sizeof *cureqs) <= 2 * alloc_lines - linbuf_base || (LineRef)(LINEREF_MAX / sizeof *linbuf) <= alloc_lines - linbuf_base)
+            if((LineRef)(LINEREF_MAX / 3) <= alloc_lines || (LineRef)(LINEREF_MAX / sizeof(*cureqs)) <= 2 * alloc_lines - linbuf_base || (LineRef)(LINEREF_MAX / sizeof(ptrdiff_t)) <= alloc_lines - linbuf_base)
                 xalloc_die();
             alloc_lines = 2 * alloc_lines - linbuf_base;
-            cureqs = (LineRef *)xrealloc(cureqs, alloc_lines * sizeof *cureqs);
+            cureqs = (LineRef *)xrealloc(cureqs, alloc_lines * sizeof(*cureqs));
             linbuf += linbuf_base;
             linbuf = (const QChar **)xrealloc(linbuf,
-                                              (alloc_lines - linbuf_base) * sizeof *linbuf);
+                                              (alloc_lines - linbuf_base) * sizeof(ptrdiff_t));
             linbuf -= linbuf_base;
         }
         linbuf[line] = ip;
@@ -275,12 +275,12 @@ void GnuDiff::find_and_hash_each_line(struct file_data *current)
         if(line == alloc_lines)
         {
             /* Double (alloc_lines - linbuf_base) by adding to alloc_lines.  */
-            if((LineRef)(LINEREF_MAX / 3) <= alloc_lines || (LineRef)(LINEREF_MAX / sizeof *cureqs) <= 2 * alloc_lines - linbuf_base || (LineRef)(LINEREF_MAX / sizeof *linbuf) <= alloc_lines - linbuf_base)
+            if((LineRef)(LINEREF_MAX / 3) <= alloc_lines || (LineRef)(LINEREF_MAX / sizeof(*cureqs)) <= 2 * alloc_lines - linbuf_base || (LineRef)(LINEREF_MAX / sizeof(ptrdiff_t)) <= alloc_lines - linbuf_base)
                 xalloc_die();
             alloc_lines = 2 * alloc_lines - linbuf_base;
             linbuf += linbuf_base;
             linbuf = (const QChar **)xrealloc(linbuf,
-                                              (alloc_lines - linbuf_base) * sizeof *linbuf);
+                                              (alloc_lines - linbuf_base) * sizeof(ptrdiff_t));
             linbuf -= linbuf_base;
         }
         linbuf[line] = p;
@@ -316,7 +316,7 @@ guess_lines(LineRef n, size_t s, size_t t)
 {
     size_t guessed_bytes_per_line = n < 10 ? 32 : s / (n - 1);
     size_t guessed_lines = std::max((size_t)1, t / guessed_bytes_per_line);
-    return (LineRef)std::min((LineRef)guessed_lines, (LineRef)(LINEREF_MAX / (2 * sizeof(QChar *) + 1) - 5)) + 5;
+    return (LineRef)std::min((LineRef)guessed_lines, (LineRef)(LINEREF_MAX / (2 * sizeof(ptrdiff_t) + 1) - 5)) + 5;
 }
 
 /* Given a vector of two file_data objects, find the identical
@@ -431,7 +431,7 @@ void GnuDiff::find_identical_ends(struct file_data filevec[])
 
     prefix_mask = prefix_count - 1;
     LineRef lines = 0;
-    linbuf0 = (const QChar **)xmalloc(alloc_lines0 * sizeof(*linbuf0));
+    linbuf0 = (const QChar **)xmalloc(alloc_lines0 * sizeof(ptrdiff_t));
     p0 = buffer0;
 
     /* If the prefix is needed, find the prefix lines.  */
@@ -443,10 +443,10 @@ void GnuDiff::find_identical_ends(struct file_data filevec[])
             LineRef l = lines++ & prefix_mask;
             if(l == alloc_lines0)
             {
-                if((LineRef)(LINEREF_MAX / (2 * sizeof *linbuf0)) <= alloc_lines0)
+                if((LineRef)(LINEREF_MAX / (2 * sizeof(ptrdiff_t))) <= alloc_lines0)
                     xalloc_die();
                 alloc_lines0 *= 2;
-                linbuf0 = (const QChar **)xrealloc(linbuf0, alloc_lines0 * sizeof(*linbuf0));
+                linbuf0 = (const QChar **)xrealloc(linbuf0, alloc_lines0 * sizeof(ptrdiff_t));
             }
             linbuf0[l] = p0;
             while(p0 < pEnd0 && !isEndOfLine(*p0++))
@@ -460,9 +460,9 @@ void GnuDiff::find_identical_ends(struct file_data filevec[])
     middle_guess = guess_lines(lines, p0 - buffer0, p1 - filevec[1].prefix_end);
     suffix_guess = guess_lines(lines, p0 - buffer0, buffer1 + n1 - p1);
     alloc_lines1 = buffered_prefix + middle_guess + std::min(context, suffix_guess);
-    if(alloc_lines1 < buffered_prefix || (LineRef)(LINEREF_MAX / sizeof *linbuf1) <= alloc_lines1)
+    if(alloc_lines1 < buffered_prefix || (LineRef)(LINEREF_MAX / sizeof(ptrdiff_t)) <= alloc_lines1)
         xalloc_die();
-    linbuf1 = (const QChar **)xmalloc(alloc_lines1 * sizeof(*linbuf1));
+    linbuf1 = (const QChar **)xmalloc(alloc_lines1 * sizeof(ptrdiff_t));
 
     LineRef i;
     if(buffered_prefix != lines)
@@ -515,9 +515,9 @@ bool GnuDiff::read_files(struct file_data filevec[], bool /*pretend_binary*/)
     find_identical_ends(filevec);
 
     equivs_alloc = filevec[0].alloc_lines + filevec[1].alloc_lines + 1;
-    if((LineRef)(LINEREF_MAX / sizeof *equivs) <= equivs_alloc)
+    if((LineRef)(LINEREF_MAX / sizeof(*equivs)) <= equivs_alloc)
         xalloc_die();
-    equivs = (equivclass *)xmalloc(equivs_alloc * sizeof *equivs);
+    equivs = (equivclass *)xmalloc(equivs_alloc * sizeof(*equivs));
     /* Equivalence class 0 is permanently safe for lines that were not
      hashed.  Real equivalence classes start at 1.  */
     equivs_index = 1;
@@ -528,9 +528,9 @@ bool GnuDiff::read_files(struct file_data filevec[], bool /*pretend_binary*/)
     for(i = 9; ((LineRef)1 << i) < equivs_alloc / 3; ++i)
         continue;
     nbuckets = ((LineRef)1 << i) - prime_offset[i];
-    if(LINEREF_MAX / sizeof *buckets <= nbuckets)
+    if(LINEREF_MAX / sizeof(*buckets) <= nbuckets)
         xalloc_die();
-    buckets = (LineRef *)zalloc((nbuckets + 1) * sizeof *buckets);
+    buckets = (LineRef *)zalloc((nbuckets + 1) * sizeof(*buckets));
     buckets++;
 
     for(i = 0; i < 2; ++i)
