@@ -379,8 +379,8 @@ class KDiff3App : public QSplitter
     bool m_bLoadFiles;
 
     void mainWindowEnable(bool bEnable);
-    virtual void wheelEvent(QWheelEvent* pWheelEvent) override;
-    virtual void keyPressEvent(QKeyEvent* event) override;
+    void wheelEvent(QWheelEvent* pWheelEvent) override;
+    void keyPressEvent(QKeyEvent* event) override;
     bool eventFilter(QObject* o, QEvent* e) override;
     void resizeEvent(QResizeEvent*) override;
 
