@@ -40,7 +40,16 @@ public:
       const Diff3LineList* pDiff3LineList,
       TotalDiffStatus* pTotalDiffStatus
       );
-   void initActions(KActionCollection* ac);
+
+   inline void clearMergeList()
+   {
+       m_mergeLineList.clear();
+       m_totalSize = 0;
+   }
+
+   static void initActions(KActionCollection* ac);
+
+   void connectActions();
    void reset();
 
    bool saveDocument( const QString& fileName, QTextCodec* pEncoding, e_LineEndStyle eLineEndStyle );
@@ -119,15 +128,15 @@ private:
    void merge(bool bAutoSolve, e_SrcSelector defaultSelector, bool bConflictsOnly=false, bool bWhiteSpaceOnly=false );
    QString getString( int lineIdx );
 
-   QAction* chooseAEverywhere = nullptr;
-   QAction* chooseBEverywhere = nullptr;
-   QAction* chooseCEverywhere = nullptr;
-   QAction* chooseAForUnsolvedConflicts = nullptr;
-   QAction* chooseBForUnsolvedConflicts = nullptr;
-   QAction* chooseCForUnsolvedConflicts = nullptr;
-   QAction* chooseAForUnsolvedWhiteSpaceConflicts = nullptr;
-   QAction* chooseBForUnsolvedWhiteSpaceConflicts = nullptr;
-   QAction* chooseCForUnsolvedWhiteSpaceConflicts = nullptr;
+   static QAction* chooseAEverywhere;
+   static QAction* chooseBEverywhere;
+   static QAction* chooseCEverywhere;
+   static QAction* chooseAForUnsolvedConflicts;
+   static QAction* chooseBForUnsolvedConflicts;
+   static QAction* chooseCForUnsolvedConflicts;
+   static QAction* chooseAForUnsolvedWhiteSpaceConflicts;
+   static QAction* chooseBForUnsolvedWhiteSpaceConflicts;
+   static QAction* chooseCForUnsolvedWhiteSpaceConflicts;
 
    Options* m_pOptions = nullptr;
 
@@ -150,17 +159,17 @@ private:
    class MergeEditLine
    {
    public:
-      explicit MergeEditLine(const Diff3LineList::const_iterator &i, e_SrcSelector src=None){m_id3l=i; m_src=src; m_bLineRemoved=false; }
-      void setConflict() { m_src=None; m_bLineRemoved=false; m_str=QString(); }
-      bool isConflict()  { return  m_src==None && !m_bLineRemoved && m_str.isEmpty(); }
-      void setRemoved(e_SrcSelector src=None)  { m_src=src; m_bLineRemoved=true; m_str=QString(); }
+      explicit MergeEditLine(const Diff3LineList::const_iterator &i, e_SrcSelector src=None){m_id3l=i; m_src=src; m_bLineRemoved=false; mChanged = false; }
+      void setConflict() { mChanged = false; m_src=None; m_bLineRemoved=false; m_str=QString(); }
+      bool isConflict()  { return  m_src==None && !m_bLineRemoved && !mChanged; }
+      void setRemoved(e_SrcSelector src=None)  { m_src=src; m_bLineRemoved=true; mChanged=(m_src == None); m_str=QString(); }
       bool isRemoved()   { return m_bLineRemoved; }
       bool isEditableText() { return !isConflict() && !isRemoved(); }
-      void setString( const QString& s ){ m_str=s; m_bLineRemoved=false; m_src=None; }
+      void setString( const QString& s ){ m_str=s; m_bLineRemoved=false; m_src=None; mChanged=true; }
       QString getString( const MergeResultWindow* );
-      bool isModified() { return ! m_str.isEmpty() ||  (m_bLineRemoved && m_src==None); }
+      bool isModified() { return mChanged; }
 
-      void setSource( e_SrcSelector src, bool bLineRemoved ) { m_src=src; m_bLineRemoved =bLineRemoved; }
+      void setSource( e_SrcSelector src, bool bLineRemoved ) { m_src=src; m_bLineRemoved = bLineRemoved; if(bLineRemoved) mChanged = true; if(m_src != None) mChanged = false; }
       e_SrcSelector src() { return m_src; }
       Diff3LineList::const_iterator id3l(){return m_id3l;}
       // getString() is implemented as MergeResultWindow::getString()
@@ -169,6 +178,8 @@ private:
      e_SrcSelector m_src; // 1, 2 or 3 for A, B or C respectively, or 0 when line is from neither source.
      QString m_str;       // String when modified by user or null-string when orig data is used.
      bool m_bLineRemoved;
+
+     bool mChanged;
    };
 
    class MergeEditLineList : private std::list<MergeEditLine>
