@@ -44,6 +44,7 @@ public:
     // Some settings are not available in the option dialog:
     QSize  m_geometry;
     QPoint m_position;
+    bool   m_bFullScreen;
     bool   m_bMaximised;
     bool   m_bShowToolBar;
     bool   m_bShowStatusBar;
@@ -149,9 +150,7 @@ public:
     QString m_DmFileAntiPattern;
     QString m_DmDirAntiPattern;
 
-    QString m_language;
     bool m_bRightToLeftLanguage;
-
     QString m_ignorableCmdLineOptions;
     bool m_bIntegrateWithClearCase;
     bool m_bEscapeKeyQuits;
