@@ -278,7 +278,7 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, bool bLoadFiles, boo
             pp.step();
             pp.setInformation(i18n("Linediff: B <-> C"));
             if(m_sd2.hasData() && m_sd3.hasData() && m_sd2.isText() && m_sd3.isText())
-            pTotalDiffStatus->bTextBEqC = m_diff3LineList.fineDiff(B, m_sd2.getLineDataForDisplay(), m_sd3.getLineDataForDisplay());
+              pTotalDiffStatus->bTextBEqC = m_diff3LineList.fineDiff(B, m_sd2.getLineDataForDisplay(), m_sd3.getLineDataForDisplay());
             pp.step();
             pp.setInformation(i18n("Linediff: A <-> C"));
             if(m_sd1.hasData() && m_sd3.hasData() && m_sd1.isText() && m_sd3.isText())
@@ -981,7 +981,7 @@ bool KDiff3App::eventFilter(QObject* o, QEvent* e)
             if(canContinue() && !urlList.isEmpty())
             {
                 raise();
-                QString filename = urlList.first().toLocalFile();
+                QString filename = Utils::urlToString(urlList.first());
                 if(o == m_pDiffTextWindow1)
                     m_sd1.setFilename(filename);
                 else if(o == m_pDiffTextWindow2)
@@ -1709,7 +1709,7 @@ void KDiff3App::recalcWordWrap(int visibleTextWidthForPrinting)
     }
     else
     {
-        //don't leave proccessing incomplete if m_diff3LineList isEmpty as when an error occures during reading.
+        //don't leave proccessing incomplete if m_diff3LineList isEmpty as when an error occurs during reading.
         slotFinishRecalcWordWrap(visibleTextWidthForPrinting);
     }
 }
