@@ -13,11 +13,13 @@
 #include "common.h"
 
 #include <QApplication>
+#include <QEventLoop>
 #include <QLabel>
 #include <QPointer>
 #include <QProgressBar>
 #include <QPushButton>
 #include <QStatusBar>
+#include <QTimer>
 #include <QThread>
 #include <QVBoxLayout>
 
@@ -259,9 +261,6 @@ void ProgressDialog::setSubRangeTransformation(double dMin, double dMax)
     pld.m_dSubRangeMax = dMax;
 }
 
-void qt_enter_modal(QWidget*);
-void qt_leave_modal(QWidget*);
-
 void ProgressDialog::enterEventLoop(KJob* pJob, const QString& jobInfo)
 {
     m_pJob = pJob;
@@ -277,13 +276,16 @@ void ProgressDialog::enterEventLoop(KJob* pJob, const QString& jobInfo)
         show();
 
     // instead of using exec() the eventloop is entered and exited often without hiding/showing the window.
-    //qt_enter_modal(this);
-    QPointer<QEventLoop> pEventLoop =  QPointer<QEventLoop>(new QEventLoop(this));
-    m_eventLoopStack.push_back(pEventLoop);
-    pEventLoop->exec(); // this function only returns after ProgressDialog::exitEventLoop() is called.
-    pEventLoop.clear();
-    m_eventLoopStack.pop_back();
-    //qt_leave_modal(this);
+    if(m_eventLoop == nullptr)
+    {
+        m_eventLoop =  QPointer<QEventLoop>(new QEventLoop(this));
+        m_eventLoop->exec(); // this function only returns after ProgressDialog::exitEventLoop() is called.
+        m_eventLoop.clear();
+    }
+    else
+    {
+        m_eventLoop->processEvents(QEventLoop::WaitForMoreEvents);
+    }
 }
 
 void ProgressDialog::exitEventLoop()
@@ -292,8 +294,8 @@ void ProgressDialog::exitEventLoop()
         killTimer(m_progressDelayTimer);
     m_progressDelayTimer = 0;
     m_pJob = nullptr;
-    if(!m_eventLoopStack.empty())
-        m_eventLoopStack.back()->exit();
+    if( m_eventLoop != nullptr)
+        m_eventLoop->exit();
 }
 
 void ProgressDialog::recalc(bool bUpdate)
@@ -344,7 +346,6 @@ void ProgressDialog::recalc(bool bUpdate)
     }
 }
 
-#include <QTimer>
 void ProgressDialog::show()
 {
     if(m_progressDelayTimer)
@@ -443,6 +444,8 @@ void ProgressDialog::cancel(e_CancelReason eCancelReason)
     {
         m_bWasCancelled = true;
         m_eCancelReason = eCancelReason;
+        if(m_eventLoop != nullptr)
+            m_eventLoop->exit(1);
     }
 }
 
